@@ -6,8 +6,8 @@ import torch
 import torch.nn as nn
 from gym.spaces import Box
 
-import raylab.pytorch.nn as nnx
-import raylab.pytorch.nn.distributions as ptd
+import raylab.torch.nn as nnx
+import raylab.torch.nn.distributions as ptd
 from raylab.policy.modules.networks.mlp import StateActionMLP
 
 
